@@ -122,8 +122,6 @@ class install(Command):
          "force installation (overwrite any existing files)"),
         ('skip-build', None,
          "skip rebuilding everything (for testing/debugging)"),
-        ('executable=', 'e',
-         "specify final destination interpreter path (install.py)"),
 
         # Where to install documentation (eventually!)
         #('doc-format=', None, "format of documentation to generate"),
@@ -197,7 +195,6 @@ class install(Command):
         # directory not in sys.path.
         self.force = 0
         self.skip_build = 0
-        self.executable = None
         self.warn_dir = 1
 
         # These are only here as a conduit from the 'build' command to the
@@ -370,9 +367,6 @@ class install(Command):
                                    ('build_base', 'build_base'),
                                    ('build_lib', 'build_lib'))
 
-        if self.executable is None:
-            self.executable = os.path.normpath(sys.executable)
-
         # Punt on doc directories for now -- after all, we're punting on
         # documentation completely!
 
@@ -424,8 +418,18 @@ class install(Command):
                     raise DistutilsOptionError(
                           "must not supply exec-prefix without prefix")
 
-                self.prefix = os.path.normpath(sys.prefix)
-                self.exec_prefix = os.path.normpath(sys.exec_prefix)
+                # self.prefix is set to sys.prefix + /local/
+                # if the executable is /usr/bin/python* and RPM build
+                # is not detected to make pip and distutils install into
+                # the separate location.
+                if (sys.executable.startswith("/usr/bin/python")
+                    and 'RPM_BUILD_ROOT' not in os.environ):
+                    addition = "/local"
+                else:
+                    addition = ""
+
+                self.prefix = os.path.normpath(sys.prefix) + addition
+                self.exec_prefix = os.path.normpath(sys.exec_prefix) + addition
 
             else:
                 if self.exec_prefix is None:
