@@ -1,7 +1,7 @@
 # updateinfo.py
 # UpdateInfo CLI command.
 #
-# Copyright (C) 2014  Red Hat, Inc.
+# Copyright (C) 2014-2016 Red Hat, Inc.
 #
 # This copyrighted material is made available to anyone wishing to use,
 # modify, copy, or redistribute it subject to the terms and conditions of
@@ -25,6 +25,7 @@ from __future__ import print_function
 from __future__ import unicode_literals
 from collections import OrderedDict
 from dnf.cli import commands
+from dnf.cli.option_parser import OptionParser
 from dnf.i18n import _
 from dnf.pycomp import unicode
 from itertools import chain
@@ -45,11 +46,25 @@ class UpdateInfoCommand(commands.Command):
     TYPE2LABEL = {hawkey.ADVISORY_BUGFIX: _('bugfix'),
                   hawkey.ADVISORY_ENHANCEMENT: _('enhancement'),
                   hawkey.ADVISORY_SECURITY: _('security'),
-                  hawkey.ADVISORY_UNKNOWN: _('unknown')}
+                  hawkey.ADVISORY_UNKNOWN: _('unknown'),
+                  hawkey.ADVISORY_NEWPACKAGE: _('newpackage')}
 
-    aliases = ['updateinfo']
+    SECURITY2LABEL = {'Critical': _('Critical/Sec.'),
+                      'Important': _('Important/Sec.'),
+                      'Moderate': _('Moderate/Sec.'),
+                      'Low': _('Low/Sec.'),
+                      None: _('Unknown/Sec.')}
+
+    direct_commands = {'list-updateinfo'    : 'list',
+                       'list-security'      : 'list',
+                       'list-sec'           : 'list',
+                       'info-updateinfo'    : 'info',
+                       'info-security'      : 'info',
+                       'info-sec'           : 'info',
+                       'summary-updateinfo' : 'summary'}
+
+    aliases = ['updateinfo'] + list(direct_commands.keys())
     summary = _('display advisories about packages')
-    usage = ''
 
     def __init__(self, cli):
         """Initialize the command."""
@@ -75,6 +90,9 @@ class UpdateInfoCommand(commands.Command):
             ievr = self._ina2evr_cache[(apackage.name, apackage.arch)]
         except KeyError:
             return False
+        q = self.base.sack.query().filter(name=apackage.name, evr=apackage.evr)
+        if len(self.base._merge_update_filters(q, warning=False)) == 0:
+            return False
         return self.base.sack.evr_cmp(ievr, apackage.evr) < 0
 
     def _newer_equal_installed(self, apkg):
@@ -86,6 +104,9 @@ class UpdateInfoCommand(commands.Command):
             ievr = self._ina2evr_cache[(apkg.name, apkg.arch)]
         except KeyError:
             return False
+        q = self.base.sack.query().filter(name=apkg.name, evr=apkg.evr)
+        if len(self.base._merge_update_filters(q, warning=False)) == 0:
+            return False
         return self.base.sack.evr_cmp(ievr, apkg.evr) >= 0
 
     def _any_installed(self, apkg):
@@ -93,12 +114,38 @@ class UpdateInfoCommand(commands.Command):
         # Non-cached lookup not implemented. Fill the cache or implement the
         # functionality via the slow sack query.
         assert self._ina2evr_cache is not None
+        q = self.base.sack.query().filter(name=apkg.name, evr=apkg.evr)
+        if len(self.base._merge_update_filters(q, warning=False)) == 0:
+            return False
         return (apkg.name, apkg.arch) in self._ina2evr_cache
 
-    def configure(self, args):
+    def _canonical(self):
+        # were we called with direct command?
+        direct = self.direct_commands.get(self.opts.command[0])
+        if direct:
+            self.opts.spec_action = direct
+
+    @staticmethod
+    def set_argparser(parser):
+        cmds = ['summary', 'list', 'info']
+        parser.add_argument('spec', nargs='*', metavar='SPEC',
+                            choices=cmds, default=cmds[0],
+                            action=OptionParser.PkgNarrowCallback)
+        output_format = parser.add_mutually_exclusive_group()
+        output_format.add_argument("--summary", dest='output_format', const='summary',
+                                   action='store_const', help=_('show summary of advisories'))
+        output_format.add_argument("--list", dest='output_format', const='list',
+                                   action='store_const', help=_('show list of advisories'))
+        output_format.add_argument("--info", dest='output_format', const='info',
+                                   action='store_const', help=_('show info of advisories'))
+
+    def configure(self):
         """Do any command-specific configuration based on command arguments."""
-        super(UpdateInfoCommand, self).configure(args)
+        self.cli.demands.available_repos = True
         self.cli.demands.sack_activation = True
+        self._canonical()
+        if self.opts.output_format:
+            self.opts.spec_action = self.opts.output_format
 
     @staticmethod
     def _apackage_advisory_match(apackage, advisory, specs=()):
@@ -115,6 +162,8 @@ class UpdateInfoCommand(commands.Command):
             types.add(hawkey.ADVISORY_ENHANCEMENT)
         if {'security', 'sec'} & specs:
             types.add(hawkey.ADVISORY_SECURITY)
+        if 'newpackage' in specs:
+            types.add(hawkey.ADVISORY_NEWPACKAGE)
 
         return (any(fnmatch.fnmatchcase(advisory.id, pat) for pat in specs) or
                 advisory.type in types or
@@ -165,28 +214,48 @@ class UpdateInfoCommand(commands.Command):
         # Remove duplicate advisory IDs. We assume that the ID is unique within
         # a repository and two advisories with the same IDs in different
         # repositories must have the same type.
-        id2type = {pkadin[1].id: pkadin[1].type for pkadin in apkg_adv_insts}
+        id2type = {}
+        for pkadin in apkg_adv_insts:
+            id2type[pkadin[1].id] = pkadin[1].type
+            if pkadin[1].type == hawkey.ADVISORY_SECURITY:
+                id2type[(pkadin[1].id, pkadin[1].severity)] = (pkadin[1].type,
+                                                               pkadin[1].severity)
         return collections.Counter(id2type.values())
 
-    @classmethod
-    def display_summary(cls, apkg_adv_insts, mixed, description):
+    def display_summary(self, apkg_adv_insts, mixed, description):
         """Display the summary of advisories."""
-        typ2cnt = cls._summary(apkg_adv_insts)
+        typ2cnt = self._summary(apkg_adv_insts)
         if not typ2cnt:
+            if self.base.conf.autocheck_running_kernel:
+                self.cli._check_running_kernel()
             return
         print(_('Updates Information Summary: ') + description)
         # Convert types to strings and order the entries.
         label_counts = [
-            (_('Security notice(s)'), typ2cnt[hawkey.ADVISORY_SECURITY]),
-            (_('Bugfix notice(s)'), typ2cnt[hawkey.ADVISORY_BUGFIX]),
-            (_('Enhancement notice(s)'), typ2cnt[hawkey.ADVISORY_ENHANCEMENT]),
-            (_('other notice(s)'), typ2cnt[hawkey.ADVISORY_UNKNOWN])]
+            (0, _('New Package notice(s)'), typ2cnt[hawkey.ADVISORY_NEWPACKAGE]),
+            (0, _('Security notice(s)'), typ2cnt[hawkey.ADVISORY_SECURITY]),
+            (1, _('Critical Security notice(s)'),
+             typ2cnt[(hawkey.ADVISORY_SECURITY, 'Critical')]),
+            (1, _('Important Security notice(s)'),
+             typ2cnt[(hawkey.ADVISORY_SECURITY, 'Important')]),
+            (1, _('Moderate Security notice(s)'),
+             typ2cnt[(hawkey.ADVISORY_SECURITY, 'Moderate')]),
+            (1, _('Low Security notice(s)'),
+             typ2cnt[(hawkey.ADVISORY_SECURITY, 'Low')]),
+            (1, _('Unknown Security notice(s)'),
+             typ2cnt[(hawkey.ADVISORY_SECURITY, None)]),
+            (0, _('Bugfix notice(s)'), typ2cnt[hawkey.ADVISORY_BUGFIX]),
+            (0, _('Enhancement notice(s)'), typ2cnt[hawkey.ADVISORY_ENHANCEMENT]),
+            (0, _('other notice(s)'), typ2cnt[hawkey.ADVISORY_UNKNOWN])]
         # Convert counts to strings and skip missing types.
-        label2value = OrderedDict(
-            (label, unicode(count)) for label, count in label_counts if count)
-        width = _maxlen(label2value.values())
-        for label, value in label2value.items():
-            print('    %*s %s' % (width, value, label))
+        label2value = OrderedDict((label, (indent, unicode(count)))
+                                  for indent, label, count in label_counts
+                                  if count)
+        width = _maxlen(v[1] for v in label2value.values())
+        for label, (indent, value) in label2value.items():
+            print('    %*s %s' % (width + 4 * indent, value, label))
+        if self.base.conf.autocheck_running_kernel:
+            self.cli._check_running_kernel()
 
     @staticmethod
     def _list(apkg_adv_insts):
@@ -194,7 +263,7 @@ class UpdateInfoCommand(commands.Command):
         # Get ((NEVRA, installed), advisory ID, advisory type)
         apkg2nevra = lambda apkg: apkg.name + '-' + apkg.evr + '.' + apkg.arch
         nevrains_id_types = (
-            ((apkg2nevra(apkg), inst), adv.id, adv.type)
+            ((apkg2nevra(apkg), inst), adv.id, (adv.type, adv.severity))
             for apkg, adv, inst in apkg_adv_insts)
         # Sort and group by (NEVRA, installed).
         nevrains_nits = itertools.groupby(
@@ -207,12 +276,18 @@ class UpdateInfoCommand(commands.Command):
     @classmethod
     def display_list(cls, apkg_adv_insts, mixed, description):
         """Display the list of advisories."""
+        def inst2mark(inst):
+            return ('' if not mixed else 'i ' if inst else '  ')
+
+        def type2label(typ, sev):
+            return (cls.SECURITY2LABEL[sev] if typ == hawkey.ADVISORY_SECURITY
+                    else cls.TYPE2LABEL[typ])
+
         # Sort IDs and convert types to labels.
-        inst2mark = lambda inst: '' if not mixed else 'i ' if inst else '  '
         nevramark2id2tlbl = OrderedDict(
             ((nevra, inst2mark(inst)),
-             OrderedDict(sorted(((id_, cls.TYPE2LABEL[typ])
-                                 for id_, typ in id2type.items()),
+             OrderedDict(sorted(((id_, type2label(typ, sev))
+                                 for id_, (typ, sev) in id2type.items()),
                                 key=itemgetter(0))))
             for (nevra, inst), id2type in cls._list(apkg_adv_insts))
         if not nevramark2id2tlbl:
@@ -250,6 +325,7 @@ class UpdateInfoCommand(commands.Command):
                     getrefs(apkg_adv_inst[1], hawkey.REFERENCE_BUGZILLA),
                     getrefs(apkg_adv_inst[1], hawkey.REFERENCE_CVE),
                     apkg_adv_inst[1].description,
+                    apkg_adv_inst[1].severity,
                     apkg_adv_inst[1].rights,
                     (pkg.filename for pkg in apkg_adv_inst[1].packages
                      if pkg.arch in self.base.sack.list_arches()),
@@ -257,8 +333,8 @@ class UpdateInfoCommand(commands.Command):
             else:
                 # If the stored advisory is marked as not installed and the
                 # current is marked as installed, mark the stored as installed.
-                if not tuple_[9] and inst:
-                    id2tuple[identity] = tuple_[:9] + (inst,)
+                if not tuple_[10] and inst:
+                    id2tuple[identity] = tuple_[:10] + (inst,)
         # Get mapping from title to (ID, type, time, BZs, CVEs, description,
         # rights, files, installed) => group by titles and merge values. We
         # assume that two advisories with the same title (e.g. from different
@@ -269,14 +345,14 @@ class UpdateInfoCommand(commands.Command):
         for tuple_ in id2tuple.values():
             title, new = tuple_[0], tuple_[1:]
             old = title2info.get(
-                title, (None, None, None, [], [], None, None, [], False))
+                title, (None, None, None, [], [], None, None, None, [], False))
             title2info[title] = (
                 new[:3] +
                 (merge(old[3], new[3]),
                  merge(old[4], new[4])) +
-                new[5:7] +
-                (merge(old[7], new[7]),
-                 old[8] or new[8]))
+                new[5:8] +
+                (merge(old[8], new[8]),
+                 old[9] or new[9]))
         return title2info
 
     def display_info(self, apkg_adv_insts, mixed, description):
@@ -286,21 +362,22 @@ class UpdateInfoCommand(commands.Command):
         verbse = lambda value: value if self.base.conf.verbose else None
         info = (
             (tit, ([id_], [self.TYPE2LABEL[typ]], [unicode(upd)],
-                   (id_title[0] + ' - ' + id_title[1] for id_title in bzs),
-                   (id_title[0] for id_title in cvs), desc.splitlines(),
+                   (id_title[0] + (' - ' + id_title[1] if id_title[1] else '')
+                    for id_title in bzs),
+                   (id_title[0] for id_title in cvs), desc.splitlines(), [sev],
                    verbse(rigs.splitlines() if rigs else None), verbse(fils),
                    None if not mixed else [_('true') if ins else _('false')]))
-             for tit, (id_, typ, upd, bzs, cvs, desc, rigs, fils, ins) in info)
+            for tit, (id_, typ, upd, bzs, cvs, desc, sev, rigs, fils, ins) in info)
         labels = (_('Update ID'), _('Type'), _('Updated'), _('Bugs'),
-                  _('CVEs'), _('Description'), _('Rights'), _('Files'),
-                  _('Installed'))
+                  _('CVEs'), _('Description'), _('Severity'), _('Rights'),
+                  _('Files'), _('Installed'))
         width = _maxlen(labels)
         for title, vallines in info:
             print('=' * 79)
             print('  ' + title)
             print('=' * 79)
             for label, lines in zip(labels, vallines):
-                if lines is None:
+                if lines is None or lines == [None]:
                     continue
                 # Use the label only for the first item. For the remaining
                 # items, use an empty label.
@@ -309,17 +386,16 @@ class UpdateInfoCommand(commands.Command):
                     print('%*s : %s' % (width, label_, line))
             print()
 
-    def run(self, args):
+    def run(self):
         """Execute the command with arguments."""
+        self.cli._populate_update_security_filter(self.opts, minimal=True)
 
-        super(UpdateInfoCommand, self).run(args)
+        args = self.opts.spec
         display = self.display_summary
-        if args[:1] in (['summary'], []):
-            args = args[1:]
-        elif args[:1] == ['list']:
-            display, args = self.display_list, args[1:]
-        elif args[:1] == ['info']:
-            display, args = self.display_info, args[1:]
+        if self.opts.spec_action == 'list':
+            display = self.display_list
+        elif self.opts.spec_action == 'info':
+            display = self.display_info
 
         self.refresh_installed_cache()
 
